Ext.define("flash.events.Event", function(Event) {
    /*package flash.events {
/**
 * The Event class is used as the base class for the creation of Event objects, which are passed as parameters to event listeners when an event occurs.
 * <p>The properties of the Event class carry basic information about an event, such as the event's type or whether the event's default behavior can be canceled. For many events, such as the events represented by the Event class constants, this basic information is sufficient. Other events, however, may require more detailed information. Events associated with a mouse click, for example, need to include additional information about the location of the click event and whether any keys were pressed during the click event. You can pass such additional information to event listeners by extending the Event class, which is what the MouseEvent class does. ActionScript 3.0 API defines several Event subclasses for common events that require additional information. Events associated with each of the Event subclasses are described in the documentation for each class.</p>
 * <p>The methods of the Event class can be used in event listener functions to affect the behavior of the event object. Some events have an associated default behavior. For example, the <code>doubleClick</code> event has an associated default behavior that highlights the word under the mouse pointer at the time of the event. Your event listener can cancel this behavior by calling the <code>preventDefault()</code> method. You can also make the current event listener the last one to process an event by calling the <code>stopPropagation()</code> or <code>stopImmediatePropagation()</code> method.</p>
 * <p>Other sources of information include:</p>
 * <ul>
 * <li>A useful description about the timing of events, code execution, and rendering at runtime in Ted Patrick's blog entry: <a href="http://www.onflex.org/ted/2005/07/flash-player-mental-model-elastic.php">Flash Player Mental Model - The Elastic Racetrack</a>.</li>
 * <li>A blog entry by Johannes Tacskovics about the timing of frame events, such as ENTER_FRAME, EXIT_FRAME: <a href="http://blog.johannest.com/2009/06/15/the-movieclip-life-cycle-revisited-from-event-added-to-event-removed_from_stage/">The MovieClip Lifecycle</a>.</li>
 * <li>An article by Trevor McCauley about the order of ActionScript operations: <a href="http://www.senocular.com/flash/tutorials/orderofoperations/">Order of Operations in ActionScript</a>.</li>
 * <li>A blog entry by Matt Przybylski on creating custom events: <a href="http://evolve.reintroducing.com/2007/10/23/as3/as3-custom-events/">AS3: Custom Events</a>.</li></ul>
 * <p><a href="http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/flash/events/Event.html#includeExamplesSummary">View the examples</a></p>
 * @see EventDispatcher
 * @see http://help.adobe.com/en_US/Flex/4.0/UsingSDK/WS2db454920e96a9e51e63e3d11c0bf64a29-7fff.html About events
 * @see http://help.adobe.com/en_US/Flex/4.0/UsingSDK/WS2db454920e96a9e51e63e3d11c0bf64a29-7ffe.html About the Event class
 * @see http://help.adobe.com/en_US/Flex/4.0/UsingSDK/WS2db454920e96a9e51e63e3d11c0bf69084-7cdf.html Using events
 * @see http://help.adobe.com/en_US/Flex/4.0/UsingSDK/WS2db454920e96a9e51e63e3d11c0bf69084-7ce1.html Manually dispatching events
 * @see http://help.adobe.com/en_US/Flex/4.0/UsingSDK/WS2db454920e96a9e51e63e3d11c0bf69084-7cdb.html Event propagation
 * @see http://help.adobe.com/en_US/Flex/4.0/UsingSDK/WS2db454920e96a9e51e63e3d11c0bf69084-7cda.html Event priorities
 * @see http://help.adobe.com/en_US/Flex/4.0/UsingSDK/WS2db454920e96a9e51e63e3d11c0bf69084-7ce0.html Using event subclasses
 * @see http://help.adobe.com/en_US/Flex/4.0/UsingSDK/WS2db454920e96a9e51e63e3d11c0bf64a29-7fdb.html About keyboard events
 * @see http://help.adobe.com/en_US/as3/dev/WS5b3ccc516d4fbf351e63e3d118a9b90204-7e55.html Event objects
 *
 * /
public class Event {
  /**
   * Indicates whether an event is a bubbling event. If the event can bubble, this value is <code>true</code>; otherwise it is <code>false</code>.
   * <p>When an event occurs, it moves through the three phases of the event flow: the capture phase, which flows from the top of the display list hierarchy to the node just before the target node; the target phase, which comprises the target node; and the bubbling phase, which flows from the node subsequent to the target node back up the display list hierarchy.</p>
   * <p>Some events, such as the <code>activate</code> and <code>unload</code> events, do not have a bubbling phase. The <code>bubbles</code> property has a value of <code>false</code> for events that do not have a bubbling phase.</p>
   * /
  public native function get bubbles():Boolean;

  /**
   * Indicates whether the behavior associated with the event can be prevented. If the behavior can be canceled, this value is <code>true</code>; otherwise it is <code>false</code>.
   * @see #preventDefault()
   *
   * /
  public native function get cancelable():Boolean;

  /**
   * The object that is actively processing the Event object with an event listener. For example, if a user clicks an OK button, the current target could be the node containing that button or one of its ancestors that has registered an event listener for that event.
   * @see http://help.adobe.com/en_US/as3/dev/WS5b3ccc516d4fbf351e63e3d118a9b90204-7e55.html Event objects
   *
   * /
  public native function get currentTarget():Object;

  /**
   * The current phase in the event flow. This property can contain the following numeric values:
   * <ul>
   * <li>The capture phase (<code>EventPhase.CAPTURING_PHASE</code>).</li>
   * <li>The target phase (<code>EventPhase.AT_TARGET</code>).</li>
   * <li>The bubbling phase (<code>EventPhase.BUBBLING_PHASE</code>).</li></ul>
   * /
  public native function get eventPhase():uint;

  /**
   * The event target. This property contains the target node. For example, if a user clicks an OK button, the target node is the display list node containing that button.
   * @see http://help.adobe.com/en_US/as3/dev/WS5b3ccc516d4fbf351e63e3d118a9b90204-7e55.html Event objects
   *
   * /
  public native function get target():Object;

  /**
   * The type of event. The type is case-sensitive.
   * @see http://help.adobe.com/en_US/as3/dev/WS5b3ccc516d4fbf351e63e3d118a9b90204-7e55.html Event objects
   *
   * /
  public native function get type():String;

  /**
   * Creates an Event object to pass as a parameter to event listeners.
   * @param type The type of the event, accessible as <code>Event.type</code>.
   * @param bubbles Determines whether the Event object participates in the bubbling stage of the event flow. The default value is <code>false</code>.
   * @param cancelable Determines whether the Event object can be canceled. The default values is <code>false</code>.
   *
   * /
  public*/
    function Event$(type, /*:String*/
    bubbles, /*:Boolean = false*/
    cancelable) /*:Boolean = false*/
    {
        switch (Math.max(arguments.length, 1)) {
            case 1:
                bubbles = false;
            case 2:
                cancelable = false;
        }
        this['type'] = type;
        this['bubbles'] = bubbles;
        this['cancelable'] = cancelable;
    }
    /*

  /**
   * Duplicates an instance of an Event subclass.
   * <p>Returns a new Event object that is a copy of the original instance of the Event object. You do not normally call <code>clone()</code>; the EventDispatcher class calls it automatically when you redispatch an event—that is, when you call <code>dispatchEvent(event)</code> from a handler that is handling <code>event</code>.</p>
   * <p>The new Event object includes all the properties of the original.</p>
   * <p>When creating your own custom Event class, you must override the inherited <code>Event.clone()</code> method in order for it to duplicate the properties of your custom class. If you do not set all the properties that you add in your event subclass, those properties will not have the correct values when listeners handle the redispatched event.</p>
   * <p>In this example, <code>PingEvent</code> is a subclass of <code>Event</code> and therefore implements its own version of <code>clone()</code>.</p>
   * <listing>
   *      class PingEvent extends Event {
   *          var URL:String;
   *
   *      public override function clone():Event {
   *               return new PingEvent(type, bubbles, cancelable, URL);
   *         }
   *      }
   *     </listing>
   * @return A new Event object that is identical to the original.
   *
   * /
  public*/
    function clone() /*:Event*/
    {
        return new Event(this.type, this.bubbles, this.cancelable);
    }
    /*

  /**
   * A utility function for implementing the <code>toString()</code> method in custom ActionScript 3.0 Event classes. Overriding the <code>toString()</code> method is recommended, but not required.
   * <pre>     class PingEvent extends Event {
   var URL:String;

   public override function toString():String {
   return formatToString("PingEvent", "type", "bubbles", "cancelable", "eventPhase", "URL");
   }
   }
   </pre>
   * @param className The name of your custom Event class. In the previous example, the <code>className</code> parameter is <code>PingEvent</code>.
   * @param rest The properties of the Event class and the properties that you add in your custom Event class. In the previous example, the <code>...arguments</code> parameter includes <code>type</code>, <code>bubbles</code>, <code>cancelable</code>, <code>eventPhase</code>, and <code>URL</code>.
   *
   * @return The name of your custom Event class and the String value of your <code>...arguments</code> parameter.
   *
   * /
  public*/
    function formatToString(className) /*:String, ...rest*/
    /*:String*/
    {
        var rest = Array.prototype.slice.call(arguments, 1);
        var sb = /*:Array*/
            [
                "[",
                className,
                " "
            ];
        for (var i = /*:uint*/
            0; i < rest.length; ++i) {
            sb.push(rest[i], "=", this[rest[i]], " ");
        }
        sb.push("]");
        return sb.join("");
    }
    /*

  /**
   * Checks whether the <code>preventDefault()</code> method has been called on the event. If the <code>preventDefault()</code> method has been called, returns <code>true</code>; otherwise, returns <code>false</code>.
   * @return If <code>preventDefault()</code> has been called, returns <code>true</code>; otherwise, returns <code>false</code>.
   *
   * @see #preventDefault()
   *
   * /
  public*/
    function isDefaultPrevented() /*:Boolean*/
    {
        return this.defaultPrevented$1;
    }
    /*

  /**
   * Cancels an event's default behavior if that behavior can be canceled.
   * <p>Many events have associated behaviors that are carried out by default. For example, if a user types a character into a text field, the default behavior is that the character is displayed in the text field. Because the <code>TextEvent.TEXT_INPUT</code> event's default behavior can be canceled, you can use the <code>preventDefault()</code> method to prevent the character from appearing.</p>
   * <p>An example of a behavior that is not cancelable is the default behavior associated with the <code>Event.REMOVED</code> event, which is generated whenever Flash Player is about to remove a display object from the display list. The default behavior (removing the element) cannot be canceled, so the <code>preventDefault()</code> method has no effect on this default behavior.</p>
   * <p>You can use the <code>Event.cancelable</code> property to check whether you can prevent the default behavior associated with a particular event. If the value of <code>Event.cancelable</code> is <code>true</code>, then <code>preventDefault()</code> can be used to cancel the event; otherwise, <code>preventDefault()</code> has no effect.</p>
   * @see #isDefaultPrevented()
   * @see #cancelable
   *
   * /
  public*/
    function preventDefault() /*:void*/
    {
        if (this.cancelable) {
            this.defaultPrevented$1 = true;
        }
    }
    /*

  /**
   * Prevents processing of any event listeners in the current node and any subsequent nodes in the event flow. This method takes effect immediately, and it affects event listeners in the current node. In contrast, the <code>stopPropagation()</code> method doesn't take effect until all the event listeners in the current node finish processing.
   * <p><b>Note:</b> This method does not cancel the behavior associated with this event; see <code>preventDefault()</code> for that functionality.</p>
   * @see #stopPropagation()
   * @see #preventDefault()
   *
   * /
  public*/
    function stopImmediatePropagation() /*:void*/
    {
        this.immediatePropagationStopped$1 = true;
    }
    /*

  /**
   * Prevents processing of any event listeners in nodes subsequent to the current node in the event flow. This method does not affect any event listeners in the current node (<code>currentTarget</code>). In contrast, the <code>stopImmediatePropagation()</code> method prevents processing of event listeners in both the current node and subsequent nodes. Additional calls to this method have no effect. This method can be called in any phase of the event flow.
   * <p><b>Note:</b> This method does not cancel the behavior associated with this event; see <code>preventDefault()</code> for that functionality.</p>
   * @see #stopImmediatePropagation()
   * @see #preventDefault()
   *
   * /
  public*/
    function stopPropagation() /*:void*/
    {
        this.propagationStopped$1 = true;
    }
    /*

  /**
   * Returns a string containing all the properties of the Event object. The string is in the following format:
   * <p><code>[Event type=<i>value</i> bubbles=<i>value</i> cancelable=<i>value</i>]</code></p>
   * @return A string containing all the properties of the Event object.
   *
   * /
  public*/
    function toString() /*:String*/
    {
        return this.formatToString("Event", "type", "bubbles", "cancelable");
    }
    /*

  /**
   * The <code>ACTIVATE</code> constant defines the value of the <code>type</code> property of an <code>activate</code> event object.
   * <p><b>Note:</b> This event has neither a "capture phase" nor a "bubble phase", which means that event listeners must be added directly to any potential targets, whether the target is on the display list or not.</p>
   * <p>AIR for TV devices never automatically dispatch this event. You can, however, dispatch it manually.</p>
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>Any DisplayObject instance with a listener registered for the <code>activate</code> event.</td></tr></table>
   * @see EventDispatcher#event:activate
   * @see #DEACTIVATE
   *
   * /
  public static const ACTIVATE:String = "activate";
  /**
   * The <code>Event.ADDED</code> constant defines the value of the <code>type</code> property of an <code>added</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>true</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The DisplayObject instance being added to the display list. The <code>target</code> is not always the object in the display list that registered the event listener. Use the <code>currentTarget</code> property to access the object in the display list that is currently processing the event.</td></tr></table>
   * @see flash.display.DisplayObject#event:added
   * @see #ADDED_TO_STAGE
   * @see #REMOVED
   * @see #REMOVED_FROM_STAGE
   *
   * /
  public static const ADDED:String = "added";
  /**
   * The <code>Event.ADDED_TO_STAGE</code> constant defines the value of the <code>type</code> property of an <code>addedToStage</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The DisplayObject instance being added to the on stage display list, either directly or through the addition of a sub tree in which the DisplayObject instance is contained. If the DisplayObject instance is being directly added, the <code>added</code> event occurs before this event.</td></tr></table>
   * @see flash.display.DisplayObject#event:addedToStage
   * @see #ADDED
   * @see #REMOVED
   * @see #REMOVED_FROM_STAGE
   *
   * /
  public static const ADDED_TO_STAGE:String = "addedToStage";
  /**
   * The <code>Event.CANCEL</code> constant defines the value of the <code>type</code> property of a <code>cancel</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>A reference to the object on which the operation is canceled.</td></tr></table>
   * @see flash.net.FileReference#event:cancel
   *
   * /
  public static const CANCEL:String = "cancel";
  /**
   * The <code>Event.CHANGE</code> constant defines the value of the <code>type</code> property of a <code>change</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>true</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The object that has had its value modified. The <code>target</code> is not always the object in the display list that registered the event listener. Use the <code>currentTarget</code> property to access the object in the display list that is currently processing the event.</td></tr></table>
   * @see flash.text.TextField#event:change
   * @see TextEvent#TEXT_INPUT
   *
   * /
  public static const CHANGE:String = "change";
  /**
   * The <code>Event.CLEAR</code> constant defines the value of the <code>type</code> property of a <code>clear</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>Any InteractiveObject instance with a listener registered for the <code>clear</code> event.</td></tr></table>
   * <p><b>Note:</b> TextField objects do <i>not</i> dispatch <code>clear</code>, <code>copy</code>, <code>cut</code>, <code>paste</code>, or <code>selectAll</code> events. TextField objects always include Cut, Copy, Paste, Clear, and Select All commands in the context menu. You cannot remove these commands from the context menu for TextField objects. For TextField objects, selecting these commands (or their keyboard equivalents) does not generate <code>clear</code>, <code>copy</code>, <code>cut</code>, <code>paste</code>, or <code>selectAll</code> events. However, other classes that extend the InteractiveObject class, including components built using the Flash Text Engine (FTE), will dispatch these events in response to user actions such as keyboard shortcuts and context menus.</p>
   * @see flash.display.InteractiveObject#event:clear
   *
   * /
  public static const CLEAR:String = "clear";
  /**
   * The <code>Event.CLOSE</code> constant defines the value of the <code>type</code> property of a <code>close</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The object whose connection has been closed.</td></tr></table>
   * @see flash.net.Socket#event:close
   * @see flash.net.XMLSocket#event:close
   * @see flash.display.NativeWindow#event:close
   *
   * /
  public static const CLOSE:String = "close";
  /**
   * The <code>Event.COMPLETE</code> constant defines the value of the <code>type</code> property of a <code>complete</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The network object that has completed loading.</td></tr></table>
   * @see flash.display.LoaderInfo#event:complete
   * @see flash.html.HTMLLoader#event:complete
   * @see flash.media.Sound#event:complete
   * @see flash.net.FileReference#event:complete
   * @see flash.net.URLLoader#event:complete
   * @see flash.net.URLStream#event:complete
   * @see http://help.adobe.com/en_US/as3/dev/WS5b3ccc516d4fbf351e63e3d118a9b90204-7cfd.html Loading external data
   *
   * /
  public static const COMPLETE:String = "complete";
  /**
   * The <code>Event.CONNECT</code> constant defines the value of the <code>type</code> property of a <code>connect</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The Socket or XMLSocket object that has established a network connection.</td></tr></table>
   * @see flash.net.Socket#event:connect
   * @see flash.net.XMLSocket#event:connect
   *
   * /
  public static const CONNECT:String = "connect";
  /**
   * Defines the value of the <code>type</code> property of a <code>copy</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>Any InteractiveObject instance with a listener registered for the <code>copy</code> event.</td></tr></table>
   * <p><b>Note:</b> TextField objects do <i>not</i> dispatch <code>clear</code>, <code>copy</code>, <code>cut</code>, <code>paste</code>, or <code>selectAll</code> events. TextField objects always include Cut, Copy, Paste, Clear, and Select All commands in the context menu. You cannot remove these commands from the context menu for TextField objects. For TextField objects, selecting these commands (or their keyboard equivalents) does not generate <code>clear</code>, <code>copy</code>, <code>cut</code>, <code>paste</code>, or <code>selectAll</code> events. However, other classes that extend the InteractiveObject class, including components built using the Flash Text Engine (FTE), will dispatch these events in response to user actions such as keyboard shortcuts and context menus.</p>
   * @see flash.display.InteractiveObject#event:copy
   *
   * /
  public static const COPY:String = "copy";
  /**
   * Defines the value of the <code>type</code> property of a <code>cut</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>Any InteractiveObject instance with a listener registered for the <code>cut</code> event.</td></tr></table>
   * <p><b>Note:</b> TextField objects do <i>not</i> dispatch <code>clear</code>, <code>copy</code>, <code>cut</code>, <code>paste</code>, or <code>selectAll</code> events. TextField objects always include Cut, Copy, Paste, Clear, and Select All commands in the context menu. You cannot remove these commands from the context menu for TextField objects. For TextField objects, selecting these commands (or their keyboard equivalents) does not generate <code>clear</code>, <code>copy</code>, <code>cut</code>, <code>paste</code>, or <code>selectAll</code> events. However, other classes that extend the InteractiveObject class, including components built using the Flash Text Engine (FTE), will dispatch these events in response to user actions such as keyboard shortcuts and context menus.</p>
   * @see flash.display.InteractiveObject#event:cut
   *
   * /
  public static const CUT:String = "cut";
  /**
   * The <code>Event.DEACTIVATE</code> constant defines the value of the <code>type</code> property of a <code>deactivate</code> event object.
   * <p><b>Note:</b> This event has neither a "capture phase" nor a "bubble phase", which means that event listeners must be added directly to any potential targets, whether the target is on the display list or not.</p>
   * <p>AIR for TV devices never automatically dispatch this event. You can, however, dispatch it manually.</p>
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>Any DisplayObject instance with a listener registered for the <code>deactivate</code> event.</td></tr></table>
   * @see EventDispatcher#event:deactivate
   * @see #ACTIVATE
   *
   * /
  public static const DEACTIVATE:String = "deactivate";
  /**
   * The <code>Event.ENTER_FRAME</code> constant defines the value of the <code>type</code> property of an <code>enterFrame</code> event object.
   * <p><b>Note:</b> This event has neither a "capture phase" nor a "bubble phase", which means that event listeners must be added directly to any potential targets, whether the target is on the display list or not.</p>
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>Any DisplayObject instance with a listener registered for the <code>enterFrame</code> event.</td></tr></table>
   * @see flash.display.DisplayObject#event:enterFrame
   *
   * /
  public static const ENTER_FRAME:String = "enterFrame";
  /**
   * The <code>Event.EXIT_FRAME</code> constant defines the value of the <code>type</code> property of an <code>exitFrame</code> event object.
   * <p><b>Note:</b> This event has neither a "capture phase" nor a "bubble phase", which means that event listeners must be added directly to any potential targets, whether the target is on the display list or not.</p>
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>Any DisplayObject instance with a listener registered for the <code>enterFrame</code> event.</td></tr></table>
   * @see flash.display.DisplayObject#event:exitFrame
   *
   * /
  public static const EXIT_FRAME:String = "exitFrame";
  /**
   * The <code>Event.FRAME_CONSTRUCTED</code> constant defines the value of the <code>type</code> property of an <code>frameConstructed</code> event object.
   * <p><b>Note:</b> This event has neither a "capture phase" nor a "bubble phase", which means that event listeners must be added directly to any potential targets, whether the target is on the display list or not.</p>
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>Any DisplayObject instance with a listener registered for the <code>frameConstructed</code> event.</td></tr></table>
   * @see flash.display.DisplayObject#event:frameConstructed
   *
   * /
  public static const FRAME_CONSTRUCTED:String = "frameConstructed";
  /**
   * The <code>Event.FULL_SCREEN</code> constant defines the value of the <code>type</code> property of a <code>fullScreen</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The Stage object.</td></tr></table>
   * @see flash.display.Stage#event:fullScreen
   *
   * /
  public static const FULLSCREEN:String = "fullScreen";
  /**
   * The <code>Event.ID3</code> constant defines the value of the <code>type</code> property of an <code>id3</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The Sound object loading the MP3 for which ID3 data is now available. The <code>target</code> is not always the object in the display list that registered the event listener. Use the <code>currentTarget</code> property to access the object in the display list that is currently processing the event.</td></tr></table>
   * @see flash.media.Sound#event:id3
   *
   * /
  public static const ID3:String = "id3";
  /**
   * The <code>Event.INIT</code> constant defines the value of the <code>type</code> property of an <code>init</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The LoaderInfo object associated with the SWF file being loaded.</td></tr></table>
   * @see flash.display.LoaderInfo#event:init
   *
   * /
  public static const INIT:String = "init";
  /**
   * The <code>Event.MOUSE_LEAVE</code> constant defines the value of the <code>type</code> property of a <code>mouseLeave</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The Stage object. The <code>target</code> is not always the object in the display list that registered the event listener. Use the <code>currentTarget</code> property to access the object in the display list that is currently processing the event.</td></tr></table>
   * @see flash.display.Stage#event:mouseLeave
   * @see MouseEvent
   *
   * /
  public static const MOUSE_LEAVE:String = "mouseLeave";
  /**
   * The <code>Event.OPEN</code> constant defines the value of the <code>type</code> property of an <code>open</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The network object that has opened a connection.</td></tr></table>
   * @see flash.display.LoaderInfo#event:open
   * @see flash.media.Sound#event:open
   * @see flash.net.FileReference#event:open
   * @see flash.net.URLLoader#event:open
   * @see flash.net.URLStream#event:open
   *
   * /
  public static const OPEN:String = "open";
  /**
   * The <code>Event.PASTE</code> constant defines the value of the <code>type</code> property of a <code>paste</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>Any InteractiveObject instance with a listener registered for the <code>paste</code> event.</td></tr></table>
   * <p><b>Note:</b> TextField objects do <i>not</i> dispatch <code>clear</code>, <code>copy</code>, <code>cut</code>, <code>paste</code>, or <code>selectAll</code> events. TextField objects always include Cut, Copy, Paste, Clear, and Select All commands in the context menu. You cannot remove these commands from the context menu for TextField objects. For TextField objects, selecting these commands (or their keyboard equivalents) does not generate <code>clear</code>, <code>copy</code>, <code>cut</code>, <code>paste</code>, or <code>selectAll</code> events. However, other classes that extend the InteractiveObject class, including components built using the Flash Text Engine (FTE), will dispatch these events in response to user actions such as keyboard shortcuts and context menus.</p>
   * @see flash.display.InteractiveObject#event:paste
   *
   * /
  public static const PASTE:String = "paste";
  /**
   * The <code>Event.REMOVED</code> constant defines the value of the <code>type</code> property of a <code>removed</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>true</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The DisplayObject instance to be removed from the display list. The <code>target</code> is not always the object in the display list that registered the event listener. Use the <code>currentTarget</code> property to access the object in the display list that is currently processing the event.</td></tr></table>
   * @see flash.display.DisplayObject#event:removed
   * @see #ADDED
   * @see #ADDED_TO_STAGE
   * @see #REMOVED_FROM_STAGE
   *
   * /
  public static const REMOVED:String = "removed";
  /**
   * The <code>Event.REMOVED_FROM_STAGE</code> constant defines the value of the <code>type</code> property of a <code>removedFromStage</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The DisplayObject instance being removed from the on stage display list, either directly or through the removal of a sub tree in which the DisplayObject instance is contained. If the DisplayObject instance is being directly removed, the <code>removed</code> event occurs before this event.</td></tr></table>
   * @see flash.display.DisplayObject#event:removedFromStage
   * @see #ADDED
   * @see #REMOVED
   * @see #ADDED_TO_STAGE
   *
   * /
  public static const REMOVED_FROM_STAGE:String = "removedFromStage";
  /**
   * The <code>Event.RENDER</code> constant defines the value of the <code>type</code> property of a <code>render</code> event object.
   * <p><b>Note:</b> This event has neither a "capture phase" nor a "bubble phase", which means that event listeners must be added directly to any potential targets, whether the target is on the display list or not.</p>
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; the default behavior cannot be canceled.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>Any DisplayObject instance with a listener registered for the <code>render</code> event.</td></tr></table>
   * @see flash.display.DisplayObject#event:render
   * @see flash.display.Stage#invalidate()
   *
   * /
  public static const RENDER:String = "render";
  /**
   * The <code>Event.RESIZE</code> constant defines the value of the <code>type</code> property of a <code>resize</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The Stage object.</td></tr></table>
   * @see flash.display.Stage#event:resize
   *
   * /
  public static const RESIZE:String = "resize";
  /**
   * The <code>Event.SCROLL</code> constant defines the value of the <code>type</code> property of a <code>scroll</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The TextField object that has been scrolled. The <code>target</code> property is not always the object in the display list that registered the event listener. Use the <code>currentTarget</code> property to access the object in the display list that is currently processing the event.</td></tr></table>
   * @see flash.text.TextField#event:scroll
   * @see flash.html.HTMLLoader#event:scroll
   *
   * /
  public static const SCROLL:String = "scroll";
  /**
   * The <code>Event.SELECT</code> constant defines the value of the <code>type</code> property of a <code>select</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The object on which an item has been selected.</td></tr></table>
   * @see flash.net.FileReference#event:select
   * @see flash.display.NativeMenu#event:select
   * @see flash.display.NativeMenuItem#event:select
   *
   * /
  public static const SELECT:String = "select";
  /**
   * The <code>Event.SELECT_ALL</code> constant defines the value of the <code>type</code> property of a <code>selectAll</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>Any InteractiveObject instance with a listener registered for the <code>selectAll</code> event.</td></tr></table>
   * <p><b>Note:</b> TextField objects do <i>not</i> dispatch <code>clear</code>, <code>copy</code>, <code>cut</code>, <code>paste</code>, or <code>selectAll</code> events. TextField objects always include Cut, Copy, Paste, Clear, and Select All commands in the context menu. You cannot remove these commands from the context menu for TextField objects. For TextField objects, selecting these commands (or their keyboard equivalents) does not generate <code>clear</code>, <code>copy</code>, <code>cut</code>, <code>paste</code>, or <code>selectAll</code> events. However, other classes that extend the InteractiveObject class, including components built using the Flash Text Engine (FTE), will dispatch these events in response to user actions such as keyboard shortcuts and context menus.</p>
   * @see flash.display.InteractiveObject#event:selectAll
   *
   * /
  public static const SELECT_ALL:String = "selectAll";
  /**
   * The <code>Event.SOUND_COMPLETE</code> constant defines the value of the <code>type</code> property of a <code>soundComplete</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The SoundChannel object in which a sound has finished playing.</td></tr></table>
   * @see flash.media.SoundChannel#event:soundComplete
   *
   * /
  public static const SOUND_COMPLETE:String = "soundComplete";
  /**
   * The <code>Event.TAB_CHILDREN_CHANGE</code> constant defines the value of the <code>type</code> property of a <code>tabChildrenChange</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>true</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The object whose tabChildren flag has changed. The <code>target</code> is not always the object in the display list that registered the event listener. Use the <code>currentTarget</code> property to access the object in the display list that is currently processing the event.</td></tr></table>
   * @see flash.display.InteractiveObject#event:tabChildrenChange
   *
   * /
  public static const TAB_CHILDREN_CHANGE:String = "tabChildrenChange";
  /**
   * The <code>Event.TAB_ENABLED_CHANGE</code> constant defines the value of the <code>type</code> property of a <code>tabEnabledChange</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>true</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The InteractiveObject whose tabEnabled flag has changed. The <code>target</code> is not always the object in the display list that registered the event listener. Use the <code>currentTarget</code> property to access the object in the display list that is currently processing the event.</td></tr></table>
   * @see flash.display.InteractiveObject#event:tabEnabledChange
   *
   * /
  public static const TAB_ENABLED_CHANGE:String = "tabEnabledChange";
  /**
   * The <code>Event.TAB_INDEX_CHANGE</code> constant defines the value of the <code>type</code> property of a <code>tabIndexChange</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>true</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The object whose tabIndex has changed. The <code>target</code> is not always the object in the display list that registered the event listener. Use the <code>currentTarget</code> property to access the object in the display list that is currently processing the event.</td></tr></table>
   * @see flash.display.InteractiveObject#event:tabIndexChange
   *
   * /
  public static const TAB_INDEX_CHANGE:String = "tabIndexChange";
  /**
   * The <code>Event.TEXT_INTERACTION_MODE_CHANGE</code> constant defines the value of the <code>type</code> property of a <code>interaction mode</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The TextField object whose interaction mode property is changed. For example on Android, one can change the interaction mode to SELECTION via context menu. The <code>target</code> property is not always the object in the display list that registered the event listener. Use the <code>currentTarget</code> property to access the object in the display list that is currently processing the event.</td></tr></table>
   * @see flash.text.TextField#event:textInteractionModeChange
   *
   * /
  public static const TEXT_INTERACTION_MODE_CHANGE:String = "textInteractionModeChange";
  /**
   * The <code>Event.UNLOAD</code> constant defines the value of the <code>type</code> property of an <code>unload</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The LoaderInfo object associated with the SWF file being unloaded or replaced.</td></tr></table>
   * @see flash.display.LoaderInfo#event:unload
   *
   * /
  public static const UNLOAD:String = "unload";

  // ************************** Jangaroo part **************************

  /**
   * @private
   * /
  public*/
    function isPropagationStopped() /*:Boolean*/
    {
        return this.propagationStopped$1;
    }
    /*

  /**
   * @private
   * /
  public*/
    function isImmediatePropagationStopped() /*:Boolean*/
    {
        return this.immediatePropagationStopped$1;
    }
    /*

  /**
   * @private
   * /
  internal*/
    function withTarget(target) /*:Object*/
    /*:Event*/
    {
        var event = /*:Event*/
            this.target ? this.clone() : this;
        event['target'] = target;
        return event;
    }
    /*

  internal*/
    function withCurrentTarget(currentTarget) /*:Object*/
    /*:Event*/
    {
        this['currentTarget'] = currentTarget;
        return this;
    }
    /*

  private var defaultPrevented:Boolean = false;
  private var propagationStopped:Boolean;
  private var immediatePropagationStopped:Boolean;
}
}

============================================== Jangaroo part ==============================================*/
    return {
        constructor: Event$,
        clone: clone,
        formatToString: formatToString,
        isDefaultPrevented: isDefaultPrevented,
        preventDefault: preventDefault,
        stopImmediatePropagation: stopImmediatePropagation,
        stopPropagation: stopPropagation,
        toString: toString,
        isPropagationStopped: isPropagationStopped,
        isImmediatePropagationStopped: isImmediatePropagationStopped,
        withTarget: withTarget,
        withCurrentTarget: withCurrentTarget,
        defaultPrevented$1: false,
        propagationStopped$1: false,
        immediatePropagationStopped$1: false,
        statics: {
            ACTIVATE: "activate",
            ADDED: "added",
            ADDED_TO_STAGE: "addedToStage",
            CANCEL: "cancel",
            CHANGE: "change",
            CLEAR: "clear",
            CLOSE: "close",
            COMPLETE: "complete",
            CONNECT: "connect",
            COPY: "copy",
            CUT: "cut",
            DEACTIVATE: "deactivate",
            ENTER_FRAME: "enterFrame",
            EXIT_FRAME: "exitFrame",
            FRAME_CONSTRUCTED: "frameConstructed",
            FULLSCREEN: "fullScreen",
            ID3: "id3",
            INIT: "init",
            MOUSE_LEAVE: "mouseLeave",
            OPEN: "open",
            PASTE: "paste",
            REMOVED: "removed",
            REMOVED_FROM_STAGE: "removedFromStage",
            RENDER: "render",
            RESIZE: "resize",
            SCROLL: "scroll",
            SELECT: "select",
            SELECT_ALL: "selectAll",
            SOUND_COMPLETE: "soundComplete",
            TAB_CHILDREN_CHANGE: "tabChildrenChange",
            TAB_ENABLED_CHANGE: "tabEnabledChange",
            TAB_INDEX_CHANGE: "tabIndexChange",
            TEXT_INTERACTION_MODE_CHANGE: "textInteractionModeChange",
            UNLOAD: "unload"
        }
    };
});

Ext.define("flash.events.IEventDispatcher", function(IEventDispatcher) {
    /*package flash.events {


/**
 * The IEventDispatcher interface defines methods for adding or removing event listeners, checks whether specific types of event listeners are registered, and dispatches events.
 * <p>Event targets are an important part of the Flash<sup>&#174;</sup> Player and Adobe AIR event model. The event target serves as the focal point for how events flow through the display list hierarchy. When an event such as a mouse click or a keypress occurs, an event object is dispatched into the event flow from the root of the display list. The event object makes a round-trip journey to the event target, which is conceptually divided into three phases: the capture phase includes the journey from the root to the last node before the event target's node; the target phase includes only the event target node; and the bubbling phase includes any subsequent nodes encountered on the return trip to the root of the display list.</p>
 * <p>In general, the easiest way for a user-defined class to gain event dispatching capabilities is to extend EventDispatcher. If this is impossible (that is, if the class is already extending another class), you can instead implement the IEventDispatcher interface, create an EventDispatcher member, and write simple hooks to route calls into the aggregated EventDispatcher.</p>
 * <p><a href="http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/flash/events/IEventDispatcher.html#includeExamplesSummary">View the examples</a></p>
 * /
public interface IEventDispatcher {
  /**
   * Registers an event listener object with an EventDispatcher object so that the listener receives notification of an event. You can register event listeners on all nodes in the display list for a specific type of event, phase, and priority.
   * <p>After you successfully register an event listener, you cannot change its priority through additional calls to <code>addEventListener()</code>. To change a listener's priority, you must first call <code>removeEventListener()</code>. Then you can register the listener again with the new priority level.</p>
   * <p>After the listener is registered, subsequent calls to <code>addEventListener()</code> with a different value for either <code>type</code> or <code>useCapture</code> result in the creation of a separate listener registration. For example, if you first register a listener with <code>useCapture</code> set to <code>true</code>, it listens only during the capture phase. If you call <code>addEventListener()</code> again using the same listener object, but with <code>useCapture</code> set to <code>false</code>, you have two separate listeners: one that listens during the capture phase, and another that listens during the target and bubbling phases.</p>
   * <p>You cannot register an event listener for only the target phase or the bubbling phase. Those phases are coupled during registration because bubbling applies only to the ancestors of the target node.</p>
   * <p>When you no longer need an event listener, remove it by calling <code>EventDispatcher.removeEventListener()</code>; otherwise, memory problems might result. Objects with registered event listeners are not automatically removed from memory because the garbage collector does not remove objects that still have references.</p>
   * <p>Copying an EventDispatcher instance does not copy the event listeners attached to it. (If your newly created node needs an event listener, you must attach the listener after creating the node.) However, if you move an EventDispatcher instance, the event listeners attached to it move along with it.</p>
   * <p>If the event listener is being registered on a node while an event is also being processed on this node, the event listener is not triggered during the current phase but may be triggered during a later phase in the event flow, such as the bubbling phase.</p>
   * <p>If an event listener is removed from a node while an event is being processed on the node, it is still triggered by the current actions. After it is removed, the event listener is never invoked again (unless it is registered again for future processing).</p>
   * @param type The type of event.
   * @param listener The listener function that processes the event. This function must accept an event object as its only parameter and must return nothing, as this example shows:
   * <p><code>function(evt:Event):void</code></p>The function can have any name.
   * @param useCapture Determines whether the listener works in the capture phase or the target and bubbling phases. If <code>useCapture</code> is set to <code>true</code>, the listener processes the event only during the capture phase and not in the target or bubbling phase. If <code>useCapture</code> is <code>false</code>, the listener processes the event only during the target or bubbling phase. To listen for the event in all three phases, call <code>addEventListener()</code> twice, once with <code>useCapture</code> set to <code>true</code>, then again with <code>useCapture</code> set to <code>false</code>.
   * @param priority The priority level of the event listener. Priorities are designated by a 32-bit integer. The higher the number, the higher the priority. All listeners with priority <i>n</i> are processed before listeners of priority <i>n-1</i>. If two or more listeners share the same priority, they are processed in the order in which they were added. The default priority is 0.
   * @param useWeakReference Determines whether the reference to the listener is strong or weak. A strong reference (the default) prevents your listener from being garbage-collected. A weak reference does not.
   * <p>Class-level member functions are not subject to garbage collection, so you can set <code>useWeakReference</code> to <code>true</code> for class-level member functions without subjecting them to garbage collection. If you set <code>useWeakReference</code> to <code>true</code> for a listener that is a nested inner function, the function will be garbge-collected and no longer persistent. If you create references to the inner function (save it in another variable) then it is not garbage-collected and stays persistent.</p>
   *
   * /
  function addEventListener(type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false):void;

  /**
   * Dispatches an event into the event flow. The event target is the EventDispatcher object upon which <code>dispatchEvent()</code> is called.
   * @param event The event object dispatched into the event flow.
   *
   * @return A value of <code>true</code> unless <code>preventDefault()</code> is called on the event, in which case it returns <code>false</code>.
   *
   * /
  function dispatchEvent(event:Event):Boolean;

  /**
   * Checks whether the EventDispatcher object has any listeners registered for a specific type of event. This allows you to determine where an EventDispatcher object has altered handling of an event type in the event flow hierarchy. To determine whether a specific event type will actually trigger an event listener, use <code>IEventDispatcher.willTrigger()</code>.
   * <p>The difference between <code>hasEventListener()</code> and <code>willTrigger()</code> is that <code>hasEventListener()</code> examines only the object to which it belongs, whereas <code>willTrigger()</code> examines the entire event flow for the event specified by the <code>type</code> parameter.</p>
   * @param type The type of event.
   *
   * @return A value of <code>true</code> if a listener of the specified type is registered; <code>false</code> otherwise.
   *
   * @see #willTrigger()
   *
   * /
  function hasEventListener(type:String):Boolean;

  /**
   * Removes a listener from the EventDispatcher object. If there is no matching listener registered with the EventDispatcher object, a call to this method has no effect.
   * @param type The type of event.
   * @param listener The listener object to remove.
   * @param useCapture Specifies whether the listener was registered for the capture phase or the target and bubbling phases. If the listener was registered for both the capture phase and the target and bubbling phases, two calls to <code>removeEventListener()</code> are required to remove both: one call with <code>useCapture</code> set to <code>true</code>, and another call with <code>useCapture</code> set to <code>false</code>.
   *
   * /
  function removeEventListener(type:String, listener:Function, useCapture:Boolean = false):void;

  /**
   * Checks whether an event listener is registered with this EventDispatcher object or any of its ancestors for the specified event type. This method returns <code>true</code> if an event listener is triggered during any phase of the event flow when an event of the specified type is dispatched to this EventDispatcher object or any of its descendants.
   * <p>The difference between <code>hasEventListener()</code> and <code>willTrigger()</code> is that <code>hasEventListener()</code> examines only the object to which it belongs, whereas <code>willTrigger()</code> examines the entire event flow for the event specified by the <code>type</code> parameter.</p>
   * @param type The type of event.
   *
   * @return A value of <code>true</code> if a listener of the specified type will be triggered; <code>false</code> otherwise.
   *
   * /
  function willTrigger(type:String):Boolean;
}
}

============================================== Jangaroo part ==============================================*/
    return {};
});

Ext.define("flash.events.EventDispatcher", function(EventDispatcher) {
    /*package flash.events {

/**
 * [broadcast event] Dispatched when the Flash Player or AIR application gains operating system focus and becomes active. This event is a broadcast event, which means that it is dispatched by all EventDispatcher objects with a listener registered for this event. For more information about broadcast events, see the DisplayObject class.
 * @eventType flash.events.Event.ACTIVATE
 * /
[Event(name="activate", type="flash.events.Event")]
/**
 * [broadcast event] Dispatched when the Flash Player or AIR application operating loses system focus and is becoming inactive. This event is a broadcast event, which means that it is dispatched by all EventDispatcher objects with a listener registered for this event. For more information about broadcast events, see the DisplayObject class.
 * @eventType flash.events.Event.DEACTIVATE
 * /
[Event(name="deactivate", type="flash.events.Event")]

/**
 * The EventDispatcher class is the base class for all classes that dispatch events. The EventDispatcher class implements the IEventDispatcher interface and is the base class for the DisplayObject class. The EventDispatcher class allows any object on the display list to be an event target and as such, to use the methods of the IEventDispatcher interface.
 * <p>Event targets are an important part of the Flash<sup>&#174;</sup> Player and Adobe<sup>&#174;</sup> AIR<sup>&#174;</sup> event model. The event target serves as the focal point for how events flow through the display list hierarchy. When an event such as a mouse click or a keypress occurs, Flash Player or the AIR application dispatches an event object into the event flow from the root of the display list. The event object then makes its way through the display list until it reaches the event target, at which point it begins its return trip through the display list. This round-trip journey to the event target is conceptually divided into three phases: the capture phase comprises the journey from the root to the last node before the event target's node, the target phase comprises only the event target node, and the bubbling phase comprises any subsequent nodes encountered on the return trip to the root of the display list.</p>
 * <p>In general, the easiest way for a user-defined class to gain event dispatching capabilities is to extend EventDispatcher. If this is impossible (that is, if the class is already extending another class), you can instead implement the IEventDispatcher interface, create an EventDispatcher member, and write simple hooks to route calls into the aggregated EventDispatcher.</p>
 * <p><a href="http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/flash/events/EventDispatcher.html#includeExamplesSummary">View the examples</a></p>
 * /
public class EventDispatcher implements IEventDispatcher {
  /**
   * Aggregates an instance of the EventDispatcher class.
   * <p>The EventDispatcher class is generally used as a base class, which means that most developers do not need to use this constructor function. However, advanced developers who are implementing the IEventDispatcher interface need to use this constructor. If you are unable to extend the EventDispatcher class and must instead implement the IEventDispatcher interface, use this constructor to aggregate an instance of the EventDispatcher class.</p>
   * @param target The target object for events dispatched to the EventDispatcher object. This parameter is used when the EventDispatcher instance is aggregated by a class that implements IEventDispatcher; it is necessary so that the containing object can be the target for events. Do not use this parameter in simple cases in which a class extends EventDispatcher.
   *
   * /
  public*/
    function EventDispatcher$(target) /*:IEventDispatcher = null*/
    {
        captureListeners_.call(this);
        listeners_.call(this);
        if (arguments.length <= 0)  {
            target = null;
        }
        
        this.target$1 = target;
    }
    /*

  /**
   * Registers an event listener object with an EventDispatcher object so that the listener receives notification of an event. You can register event listeners on all nodes in the display list for a specific type of event, phase, and priority.
   * <p>After you successfully register an event listener, you cannot change its priority through additional calls to <code>addEventListener()</code>. To change a listener's priority, you must first call <code>removeListener()</code>. Then you can register the listener again with the new priority level.</p>
   * <p>Keep in mind that after the listener is registered, subsequent calls to <code>addEventListener()</code> with a different <code>type</code> or <code>useCapture</code> value result in the creation of a separate listener registration. For example, if you first register a listener with <code>useCapture</code> set to <code>true</code>, it listens only during the capture phase. If you call <code>addEventListener()</code> again using the same listener object, but with <code>useCapture</code> set to <code>false</code>, you have two separate listeners: one that listens during the capture phase and another that listens during the target and bubbling phases.</p>
   * <p>You cannot register an event listener for only the target phase or the bubbling phase. Those phases are coupled during registration because bubbling applies only to the ancestors of the target node.</p>
   * <p>If you no longer need an event listener, remove it by calling <code>removeEventListener()</code>, or memory problems could result. Event listeners are not automatically removed from memory because the garbage collector does not remove the listener as long as the dispatching object exists (unless the <code>useWeakReference</code> parameter is set to <code>true</code>).</p>
   * <p>Copying an EventDispatcher instance does not copy the event listeners attached to it. (If your newly created node needs an event listener, you must attach the listener after creating the node.) However, if you move an EventDispatcher instance, the event listeners attached to it move along with it.</p>
   * <p>If the event listener is being registered on a node while an event is being processed on this node, the event listener is not triggered during the current phase but can be triggered during a later phase in the event flow, such as the bubbling phase.</p>
   * <p>If an event listener is removed from a node while an event is being processed on the node, it is still triggered by the current actions. After it is removed, the event listener is never invoked again (unless registered again for future processing).</p>
   * @param type The type of event.
   * @param listener The listener function that processes the event. This function must accept an Event object as its only parameter and must return nothing, as this example shows:
   * <listing>
   * function(evt:Event):void</listing>
   * <p>The function can have any name.</p>
   * @param useCapture Determines whether the listener works in the capture phase or the target and bubbling phases. If <code>useCapture</code> is set to <code>true</code>, the listener processes the event only during the capture phase and not in the target or bubbling phase. If <code>useCapture</code> is <code>false</code>, the listener processes the event only during the target or bubbling phase. To listen for the event in all three phases, call <code>addEventListener</code> twice, once with <code>useCapture</code> set to <code>true</code>, then again with <code>useCapture</code> set to <code>false</code>.
   * @param priority The priority level of the event listener. The priority is designated by a signed 32-bit integer. The higher the number, the higher the priority. All listeners with priority <i>n</i> are processed before listeners of priority <i>n</i>-1. If two or more listeners share the same priority, they are processed in the order in which they were added. The default priority is 0.
   * @param useWeakReference Determines whether the reference to the listener is strong or weak. A strong reference (the default) prevents your listener from being garbage-collected. A weak reference does not.
   * <p>Class-level member functions are not subject to garbage collection, so you can set <code>useWeakReference</code> to <code>true</code> for class-level member functions without subjecting them to garbage collection. If you set <code>useWeakReference</code> to <code>true</code> for a listener that is a nested inner function, the function will be garbage-collected and no longer persistent. If you create references to the inner function (save it in another variable) then it is not garbage-collected and stays persistent.</p>
   *
   * @throws ArgumentError The <code>listener</code> specified is not a function.
   *
   * @see http://help.adobe.com/en_US/as3/dev/WS5b3ccc516d4fbf351e63e3d118a9b90204-7e54.html Event listeners
   *
   * /
  public*/
    function addEventListener(type, /*:String*/
    listener, /*:Function*/
    useCapture, /*:Boolean = false*/
    priority, /*:int = 0*/
    useWeakReference) /*:Boolean = false*/
    /*:void*/
    {
        switch (Math.max(arguments.length, 2)) {
            case 2:
                useCapture = false;
            case 3:
                priority = 0;
            case 4:
                useWeakReference = false;
        }
        var listenersByType = /*:Object*/
            useCapture ? this.captureListeners$1 : this.listeners$1;
        var eventObj = /*:Object*/
            {
                type: type,
                method: listener,
                useCapture: useCapture,
                priority: priority,
                useWeakReference: useWeakReference
            };
        if (!(type in listenersByType)) {
            listenersByType[type] = [
                eventObj
            ];
        } else {
            listenersByType[type].push(eventObj);
        }
        listenersByType[type].sort(eventCompare$static);
    }
    /*

  /**
   * Dispatches an event into the event flow. The event target is the EventDispatcher object upon which the <code>dispatchEvent()</code> method is called.
   * @param event The Event object that is dispatched into the event flow. If the event is being redispatched, a clone of the event is created automatically. After an event is dispatched, its <code>target</code> property cannot be changed, so you must create a new copy of the event for redispatching to work.
   *
   * @return A value of <code>true</code> if the event was successfully dispatched. A value of <code>false</code> indicates failure or that <code>preventDefault()</code> was called on the event.
   *
   * @throws Error The event dispatch recursion limit has been reached.
   *
   * /
  public*/
    function dispatchEvent(event) /*:Event*/
    /*:Boolean*/
    {
        event = event.withTarget(this.target$1 || this);
        var ancestors = /*:Array*/
            this.createAncestorChain();
        // Capture from the top down.
        event['eventPhase'] = flash.events.EventPhase.CAPTURING_PHASE;
        internalHandleCapture$static(event, ancestors);
        // Be sure we're allowed to continue.
        if (!event.isPropagationStopped()) {
            // Handle it here, at the target.
            event['eventPhase'] = flash.events.EventPhase.AT_TARGET;
            event.withCurrentTarget(event.target);
            var listeners = /*:Array*/
                this.listeners$1[event.type];
            if (listeners) {
                processListeners$static(event, listeners);
            }
        }
        // Be sure we're allowed to continue.
        if (event.bubbles && !event.isPropagationStopped()) {
            // Bubble it back up the display chain.
            event['eventPhase'] = flash.events.EventPhase.BUBBLING_PHASE;
            internalHandleBubble$static(event, ancestors);
        }
        return !event.isDefaultPrevented();
    }
    /*

  /**
   * Checks whether the EventDispatcher object has any listeners registered for a specific type of event. This allows you to determine where an EventDispatcher object has altered handling of an event type in the event flow hierarchy. To determine whether a specific event type actually triggers an event listener, use <code>willTrigger()</code>.
   * <p>The difference between <code>hasEventListener()</code> and <code>willTrigger()</code> is that <code>hasEventListener()</code> examines only the object to which it belongs, whereas <code>willTrigger()</code> examines the entire event flow for the event specified by the <code>type</code> parameter.</p>
   * <p>When <code>hasEventListener()</code> is called from a LoaderInfo object, only the listeners that the caller can access are considered.</p>
   * @param type The type of event.
   *
   * @return A value of <code>true</code> if a listener of the specified type is registered; <code>false</code> otherwise.
   *
   * @see #willTrigger()
   *
   * /
  public*/
    function hasEventListener(type) /*:String*/
    /*:Boolean*/
    {
        return this.listeners$1[type] || this.captureListeners$1[type];
    }
    /*

  /**
   * Removes a listener from the EventDispatcher object. If there is no matching listener registered with the EventDispatcher object, a call to this method has no effect.
   * @param type The type of event.
   * @param listener The listener object to remove.
   * @param useCapture Specifies whether the listener was registered for the capture phase or the target and bubbling phases. If the listener was registered for both the capture phase and the target and bubbling phases, two calls to <code>removeEventListener()</code> are required to remove both, one call with <code>useCapture()</code> set to <code>true</code>, and another call with <code>useCapture()</code> set to <code>false</code>.
   *
   * @see http://help.adobe.com/en_US/as3/dev/WS5b3ccc516d4fbf351e63e3d118a9b90204-7e54.html Event listeners
   *
   * /
  public*/
    function removeEventListener(type, /*:String*/
    listener, /*:Function*/
    useCapture) /*:Boolean = false*/
    /*:void*/
    {
        if (arguments.length <= 2)  {
            useCapture = false;
        }
        
        var listenersByType = /*:Object*/
            useCapture ? this.captureListeners$1 : this.listeners$1;
        var listeners = /*:Array*/
            listenersByType[type];
        if (listeners) {
            for (var i = /*:int*/
                0; i < listeners.length; ++i) {
                if (listeners[i].method == listener) {
                    if (listeners.length == 1) {
                        delete listenersByType[type];
                    } else {
                        listeners.splice(i, 1);
                    }
                    break;
                }
            }
        }
    }
    /*

  /**
   * Checks whether an event listener is registered with this EventDispatcher object or any of its ancestors for the specified event type. This method returns <code>true</code> if an event listener is triggered during any phase of the event flow when an event of the specified type is dispatched to this EventDispatcher object or any of its descendants.
   * <p>The difference between the <code>hasEventListener()</code> and the <code>willTrigger()</code> methods is that <code>hasEventListener()</code> examines only the object to which it belongs, whereas the <code>willTrigger()</code> method examines the entire event flow for the event specified by the <code>type</code> parameter.</p>
   * <p>When <code>willTrigger()</code> is called from a LoaderInfo object, only the listeners that the caller can access are considered.</p>
   * @param type The type of event.
   *
   * @return A value of <code>true</code> if a listener of the specified type will be triggered; <code>false</code> otherwise.
   *
   * /
  public*/
    function willTrigger(type) /*:String*/
    /*:Boolean*/
    {
        return this.hasEventListener(type);
    }
    /*

  public*/
    function toString() /*:String*/
    {
        return [
            "EventDispatcher[target=",
            this.target$1,
            "]"
        ].join("");
    }
    /*

  // Move to a proper namespace (event_internal maybe) when namespaces are implemented.
  public*/
    function processCapture(event) /*:Event*/
    /*:void*/
    {
        event.withCurrentTarget(this.target$1 || this);
        var listeners = /*:Array*/
            this.captureListeners$1[event.type];
        if (listeners) {
            processListeners$static(event, listeners);
        }
    }
    /*

  // Move to a proper namespace (event_internal maybe) when namespaces are implemented.
  public*/
    function processBubble(event) /*:Event*/
    /*:void*/
    {
        event.withCurrentTarget(this.target$1 || this);
        var listeners = /*:Array*/
            this.listeners$1[event.type];
        if (listeners) {
            processListeners$static(event, listeners);
        }
    }
    /*

  // Internal Methods

  private static*/
    function eventCompare$static(item1, /*:Object*/
    item2) /*:Object*/
    /*:int*/
    {
        if (item1.priority > item2.priority) {
            return -1;
        } else if (item1.priority < item2.priority) {
            return 1;
        } else {
            return 0;
        }
    }
    /*

  /**
   * @private
   * /
  public*/
    function createAncestorChain() /*:Array*/
    {
        return null;
    }
    /*

  private static*/
    function internalHandleCapture$static(event, /*:Event*/
    ancestors) /*:Array*/
    /*:void*/
    {
        if (!ancestors || ancestors.length <= 0) {
            return;
        }
        // Start at the top of the chain and go down.
        for (var i = /*:int*/
            ancestors.length - 1; i >= 0; i--) {
            var d = /*:EventDispatcher*/
                ancestors[i];
            d.processCapture(event);
            if (event.isPropagationStopped()) {
                break;
            }
        }
    }
    /*

  private static*/
    function internalHandleBubble$static(event, /*:Event*/
    ancestors) /*:Array*/
    /*:void*/
    {
        if (!ancestors || ancestors.length <= 0) {
            return;
        }
        // Start at the top of the chain and go down.
        for (var i = /*:int*/
            0; i < ancestors.length; i++) {
            var d = /*:EventDispatcher*/
                ancestors[i];
            d.processBubble(event);
            if (event.isPropagationStopped()) {
                break;
            }
        }
    }
    /*

  private static*/
    function processListeners$static(event, /*:Event*/
    listeners) /*:Array*/
    /*:void*/
    {
        for (var i = /*:int*/
            0; i < listeners.length; ++i) {
            if (listeners[i].method(event) === false) {
                event.stopPropagation();
                event.preventDefault();
            }
            if (event.isImmediatePropagationStopped()) {
                break;
            }
        }
    }
    /*

  // Variables

  private var captureListeners:Object =*/
    function captureListeners_() {
        this.captureListeners$1 = ({});
    }
    /*; /*<String,Array>* /
  private var listeners:Object =*/
    function listeners_() {
        this.listeners$1 = ({});
    }
    /*; /*<String,Array>* /
  private var target:IEventDispatcher;
}
}

============================================== Jangaroo part ==============================================*/
    return {
        mixins: [
            "flash.events.IEventDispatcher"
        ],
        metadata: {
            "": [
                "Event",
                [
                    "name",
                    "activate",
                    "type",
                    "flash.events.Event"
                ],
                "Event",
                [
                    "name",
                    "deactivate",
                    "type",
                    "flash.events.Event"
                ]
            ]
        },
        constructor: EventDispatcher$,
        addEventListener: addEventListener,
        dispatchEvent: dispatchEvent,
        hasEventListener: hasEventListener,
        removeEventListener: removeEventListener,
        willTrigger: willTrigger,
        toString: toString,
        processCapture: processCapture,
        processBubble: processBubble,
        createAncestorChain: createAncestorChain,
        target$1: null,
        requires: [
            "flash.events.IEventDispatcher"
        ],
        uses: [
            "flash.events.EventPhase"
        ]
    };
});

Ext.define("flash.events.EventPhase", function(EventPhase) {
    /*package flash.events {


/**
 * The EventPhase class provides values for the <code>eventPhase</code> property of the Event class.
 * @see Event
 * @see EventDispatcher
 *
 * /
public final class EventPhase {
  /**
   * The target phase, which is the second phase of the event flow.
   * /
  public static const AT_TARGET:uint = 2;
  /**
   * The bubbling phase, which is the third phase of the event flow.
   * /
  public static const BUBBLING_PHASE:uint = 3;
  /**
   * The capturing phase, which is the first phase of the event flow.
   * /
  public static const CAPTURING_PHASE:uint = 1;
}*/
    function EventPhase$() {}
    /*
}

============================================== Jangaroo part ==============================================*/
    return {
        constructor: EventPhase$,
        statics: {
            AT_TARGET: 2,
            BUBBLING_PHASE: 3,
            CAPTURING_PHASE: 1
        }
    };
});

Ext.define("flash.events.TimerEvent", function(TimerEvent) {
    /*package flash.events {
/**
 * A Timer object dispatches a TimerEvent objects whenever the Timer object reaches the interval specified by the <code>Timer.delay</code> property.
 * <p><a href="http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/flash/events/TimerEvent.html#includeExamplesSummary">View the examples</a></p>
 * @see flash.utils.Timer
 *
 * /
public class TimerEvent extends Event {
  /**
   * Creates an Event object with specific information relevant to <code>timer</code> events. Event objects are passed as parameters to event listeners.
   * @param type The type of the event. Event listeners can access this information through the inherited <code>type</code> property.
   * @param bubbles Determines whether the Event object bubbles. Event listeners can access this information through the inherited <code>bubbles</code> property.
   * @param cancelable Determines whether the Event object can be canceled. Event listeners can access this information through the inherited <code>cancelable</code> property.
   *
   * /
  public*/
    function TimerEvent$(type, /*:String*/
    bubbles, /*:Boolean = false*/
    cancelable) /*:Boolean = false*/
    {
        switch (Math.max(arguments.length, 1)) {
            case 1:
                bubbles = false;
            case 2:
                cancelable = false;
        }
        flash.events.Event.prototype.constructor.call(this, type, bubbles, cancelable);
    }
    /*

  /**
   * Creates a copy of the TimerEvent object and sets each property's value to match that of the original.
   * @return A new TimerEvent object with property values that match those of the original.
   *
   * /
  override public*/
    function clone() /*:Event*/
    {
        return new TimerEvent(this.type, this.bubbles, this.cancelable);
    }
    /*

  /**
   * Returns a string that contains all the properties of the TimerEvent object. The string is in the following format:
   * <p><code>[TimerEvent type=<i>value</i> bubbles=<i>value</i> cancelable=<i>value</i>]</code></p>
   * @return A string that contains all the properties of the TimerEvent object.
   *
   * /
  override public*/
    function toString() /*:String*/
    {
        return this.formatToString("TimerEvent", "type", "bubbles", "cancelable");
    }
    /*

  /**
   * Instructs Flash Player or the AIR runtime to render after processing of this event completes, if the display list has been modified.
   * @example The following is an example for the <code>TimerEvent.updateAfterEvent()</code> method.
   * <listing>
   * function onTimer(event:TimerEvent):void {
   *     if (40 &lt; my_mc.x &amp;&amp; my_mc.x &lt; 375) {
   *         my_mc.x-= 50;
   *     } else {
   *         my_mc.x=374;
   *     }
   *     event.updateAfterEvent();
   * }
   *
   * var moveTimer:Timer=new Timer(50,250);
   * moveTimer.addEventListener(TimerEvent.TIMER,onTimer);
   * moveTimer.start();
   * </listing>
   * /
  public*/
    function updateAfterEvent() /*:void*/
    {}
    // TODO
    /*

  /**
   * Defines the value of the <code>type</code> property of a <code>timer</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The Timer object that has reached its interval.</td></tr></table>
   * @see flash.utils.Timer#event:timer
   *
   * /
  public static const TIMER:String = "timer";
  /**
   * Defines the value of the <code>type</code> property of a <code>timerComplete</code> event object.
   * <p>This event has the following properties:</p>
   * <table>
   * <tr><th>Property</th><th>Value</th></tr>
   * <tr>
   * <td><code>bubbles</code></td>
   * <td><code>false</code></td></tr>
   * <tr>
   * <td><code>cancelable</code></td>
   * <td><code>false</code>; there is no default behavior to cancel.</td></tr>
   * <tr>
   * <td><code>currentTarget</code></td>
   * <td>The object that is actively processing the Event object with an event listener.</td></tr>
   * <tr>
   * <td><code>target</code></td>
   * <td>The Timer object that has completed its requests.</td></tr></table>
   * @see flash.utils.Timer#event:timerComplete
   *
   * /
  public static const TIMER_COMPLETE:String = "timerComplete";
}
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "flash.events.Event",
        constructor: TimerEvent$,
        clone: clone,
        toString: toString,
        updateAfterEvent: updateAfterEvent,
        statics: {
            TIMER: "timer",
            TIMER_COMPLETE: "timerComplete"
        },
        requires: [
            "flash.events.Event"
        ]
    };
});

Ext.define("flash.utils.Dictionary", function(Dictionary) {
    /*package flash.utils {


/**
 * The Dictionary class lets you create a dynamic collection of properties, which uses strict equality (<code>===</code>) for key comparison. When an object is used as a key, the object's identity is used to look up the object, and not the value returned from calling <code>toString()</code> on it.
 * <p>The following statements show the relationship between a Dictionary object and a key object:</p>
 * <pre> var dict = new Dictionary();
 var obj = new Object();
 var key:Object = new Object();
 key.toString = function() { return "key" }

 dict[key] = "Letters";
 obj["key"] = "Letters";

 dict[key] == "Letters"; // true
 obj["key"] == "Letters"; // true
 obj[key] == "Letters"; // true because key == "key" is true b/c key.toString == "key"
 dict["key"] == "Letters"; // false because "key" === key is false
 delete dict[key]; //removes the key
 </pre>
 * <p><b>Note:</b> Objects of the type QName are not valid values for keys in Dictionary object instances. Using a QName data type for a key generates a Reference error.</p>
 * @see operators#strict_equality
 *
 * /
public dynamic class Dictionary {
  /**
   * Creates a new Dictionary object. To remove a key from a Dictionary object, use the <code>delete</code> operator.
   * @param weakKeys Instructs the Dictionary object to use "weak" references on object keys. If the only reference to an object is in the specified Dictionary object, the key is eligible for garbage collection and is removed from the table when the object is collected.
   *
   * /
  public*/
    function Dictionary$(weakKeys) /*:Boolean = false*/
    {
        if (arguments.length <= 0)  {
            weakKeys = false;
        }
        
        if (weakKeys) {
            AS3.trace("[WARN]", "Dictionary with weakKeys not supported by Jangaroo.");
        }
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        constructor: Dictionary$,
        requires: [
            "AS3.trace"
        ]
    };
});

Ext.define("flash.utils.Timer", function(Timer) {
    /*package flash.utils {
import flash.events.EventDispatcher;
import flash.events.TimerEvent;

/**
 * Dispatched whenever a Timer object reaches an interval specified according to the <code>Timer.delay</code> property.
 * @eventType flash.events.TimerEvent.TIMER
 * /
[Event(name="timer", type="flash.events.TimerEvent")]
/**
 * Dispatched whenever it has completed the number of requests set by <code>Timer.repeatCount</code>.
 * @eventType flash.events.TimerEvent.TIMER_COMPLETE
 * /
[Event(name="timerComplete", type="flash.events.TimerEvent")]

/**
 * The Timer class is the interface to timers, which let you run code on a specified time sequence. Use the <code>start()</code> method to start a timer. Add an event listener for the <code>timer</code> event to set up code to be run on the timer interval.
 * <p>You can create Timer objects to run once or repeat at specified intervals to execute code on a schedule. Depending on the SWF file's framerate or the runtime environment (available memory and other factors), the runtime may dispatch events at slightly offset intervals. For example, if a SWF file is set to play at 10 frames per second (fps), which is 100 millisecond intervals, but your timer is set to fire an event at 80 milliseconds, the event will be dispatched close to the 100 millisecond interval. Memory-intensive scripts may also offset the events.</p>
 * <p><a href="http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/flash/utils/Timer.html#includeExamplesSummary">View the examples</a></p>
 * @see http://help.adobe.com/en_US/as3/dev/WS5b3ccc516d4fbf351e63e3d118a9b90204-7f0f.html Controlling time intervals
 * @see http://help.adobe.com/en_US/as3/dev/WS5b3ccc516d4fbf351e63e3d118a9b90204-7f07.html The Timer class
 *
 * /
public class Timer extends EventDispatcher {
  /**
   * The total number of times the timer has fired since it started at zero. If the timer has been reset, only the fires since the reset are counted.
   * /
  public*/
    function get$currentCount() /*:int*/
    {
        return this._currentCount$2;
    }
    /*

  /**
   * The delay, in milliseconds, between timer events. If you set the delay interval while the timer is running, the timer will restart at the same <code>repeatCount</code> iteration.
   * <p><b>Note:</b> A <code>delay</code> lower than 20 milliseconds is not recommended. Timer frequency is limited to 60 frames per second, meaning a delay lower than 16.6 milliseconds causes runtime problems.</p>
   * @throws Error Throws an exception if the delay specified is negative or not a finite number.
   *
   * /
  public*/
    function get$delay() /*:Number*/
    {
        return this._delay$2;
    }
    /*

  /**
   * @private
   * /
  public*/
    function set$delay(value) /*:Number*/
    /*:void*/
    {
        this._delay$2 = value;
        if (this.timer$2) {
            this.stop();
            this.start();
        }
    }
    /*

  /**
   * The total number of times the timer is set to run. If the repeat count is set to 0, the timer continues forever or until the <code>stop()</code> method is invoked or the program stops. If the repeat count is nonzero, the timer runs the specified number of times. If <code>repeatCount</code> is set to a total that is the same or less then <code>currentCount</code> the timer stops and will not fire again.
   * /
  public*/
    function get$repeatCount() /*:int*/
    {
        return this._repeatCount$2;
    }
    /*

  /**
   * @private
   * /
  public*/
    function set$repeatCount(value) /*:int*/
    /*:void*/
    {
        this._repeatCount$2 = value;
        this.checkComplete$2();
    }
    /*

  /**
   * The timer's current state; <code>true</code> if the timer is running, otherwise <code>false</code>.
   * /
  public*/
    function get$running() /*:Boolean*/
    {
        return this.timer$2 != 0;
    }
    /*

  /**
   * Constructs a new Timer object with the specified <code>delay</code> and <code>repeatCount</code> states.
   * <p>The timer does not start automatically; you must call the <code>start()</code> method to start it.</p>
   * @param delay The delay between timer events, in milliseconds. A <code>delay</code> lower than 20 milliseconds is not recommended. Timer frequency is limited to 60 frames per second, meaning a delay lower than 16.6 milliseconds causes runtime problems.
   * @param repeatCount Specifies the number of repetitions. If zero, the timer repeats infinitely. If nonzero, the timer runs the specified number of times and then stops.
   *
   * @throws Error if the delay specified is negative or not a finite number
   *
   * @example In the following example, the user is given 90 seconds to enter a response in an input text field. Also, every 30 seconds, a status message lets the user know how many seconds are left.
   * <p>A Timer object is created that starts in 30 seconds (delay is set to 30000 milliseconds) and repeats three times, for a total of 90 seconds. (The timer stops after the third time.)</p>
   * <p>Two event listeners are added for the <code>myTimer</code> timer. The first is triggered by the <code>TimerEvent.TIMER</code> event, which occurs every time the timer is started. The <code>timerHandler()</code> method changes the text for the <code>statusTextField</code> text field to reflect the seconds remaining.</p>
   * <p><b>Note:</b> The Timer class keeps track of the number of times it has to start (<code>repeats</code>) by increasing the number in the <code>currentCount</code> property.)</p>
   * <p>After the timer is called for the last time, the <code>TimerEvent.TIMER_COMPLETE</code> event is dispatched and the <code>completeHandler()</code> method is called. The <code>completeHandler()</code> method changes the type of the <code>inputTextField</code> text field from <code>INPUT</code> to <code>DYNAMIC</code>, which means the user can no longer enter or change text.</p>
   * <listing>
   * package {
   *     import flash.display.Sprite;
   *     import flash.text.TextField;
   *     import flash.text.TextFieldType;
   *     import flash.text.TextFieldAutoSize;
   *     import flash.utils.Timer;
   *     import flash.events.TimerEvent;
   *     import flash.events.Event;
   *
   *     public class Timer_constructorExample extends Sprite {
   *             private var statusTextField:TextField = new TextField();
   *             private var inputTextField:TextField = new TextField();
   *             private var delay:uint = 30000;
   *             private var repeat:uint = 3;
   *             private var myTimer:Timer = new Timer(delay, repeat);
   *
   *         public function Timer_constructorExample() {
   *             inputTextField.x = 10;
   *             inputTextField.y = 10;
   *             inputTextField.border = true;
   *             inputTextField.background = true;
   *             inputTextField.height = 200;
   *             inputTextField.width = 200;
   *             inputTextField.multiline = true;
   *             inputTextField.wordWrap = true;
   *             inputTextField.type = TextFieldType.INPUT;
   *
   *             statusTextField.x = 10;
   *             statusTextField.y = 220;
   *             statusTextField.background = true;
   *             statusTextField.autoSize = TextFieldAutoSize.LEFT;
   *
   *             myTimer.start();
   *             statusTextField.text = "You have " + ((delay * repeat) / 1000)
   *                                  + " seconds to enter your response.";
   *
   *             myTimer.addEventListener(TimerEvent.TIMER, timerHandler);
   *             myTimer.addEventListener(TimerEvent.TIMER_COMPLETE, completeHandler);
   *
   *             addChild(inputTextField);
   *             addChild(statusTextField);
   *         }
   *
   *         private function timerHandler(e:TimerEvent):void{
   *             repeat--;
   *             statusTextField.text = ((delay * repeat) / 1000) + " seconds left.";
   *         }
   *
   *         private function completeHandler(e:TimerEvent):void {
   *             statusTextField.text = "Times Up.";
   *             inputTextField.type = TextFieldType.DYNAMIC;
   *         }
   *     }
   * }
   * </listing>
   * /
  public*/
    function Timer$(delay, /*:Number*/
    repeatCount) /*:int = 0*/
    {
        flash.events.EventDispatcher.prototype.constructor.call(this);
        if (arguments.length <= 1)  {
            repeatCount = 0;
        }
        
        this._delay$2 = delay;
        this._repeatCount$2 = repeatCount;
    }
    /*

  /**
   * Stops the timer, if it is running, and sets the <code>currentCount</code> property back to 0, like the reset button of a stopwatch. Then, when <code>start()</code> is called, the timer instance runs for the specified number of repetitions, as set by the <code>repeatCount</code> value.
   * @see #stop()
   *
   * /
  public*/
    function reset() /*:void*/
    {
        this.stop();
        this._currentCount$2 = 0;
    }
    /*

  /**
   * Starts the timer, if it is not already running.
   * /
  public*/
    function start() /*:void*/
    {
        if (!this.timer$2) {
            this.timer$2 = flash.utils.setInterval(AS3.bind(this, "tick$2"), this._delay$2);
        }
    }
    /*

  /**
   * Stops the timer. When <code>start()</code> is called after <code>stop()</code>, the timer instance runs for the <i>remaining</i> number of repetitions, as set by the <code>repeatCount</code> property.
   * @see #reset()
   *
   * /
  public*/
    function stop() /*:void*/
    {
        if (this.timer$2) {
            flash.utils.clearInterval(this.timer$2);
            this.timer$2 = 0;
        }
    }
    /*

  private*/
    function tick() /*:void*/
    {
        if (!this.timer$2) {
            // oops, a tick occurred although timer has been stopped:
            return;
        }
        ++this._currentCount$2;
        try {
            this.dispatchEvent(new flash.events.TimerEvent(flash.events.TimerEvent.TIMER));
        } catch (e) /*:**/
        {
            AS3.trace("ERROR", e);
        }
        this.checkComplete$2();
    }
    /*

  private*/
    function checkComplete() /*:void*/
    {
        if (this._repeatCount$2 > 0 && this._currentCount$2 >= this._repeatCount$2) {
            this.stop();
            this.dispatchEvent(new flash.events.TimerEvent(flash.events.TimerEvent.TIMER_COMPLETE));
        }
    }
    /*

  private var timer:uint = 0;
  private var _delay:Number;
  private var _repeatCount:int;
  private var _currentCount:int = 0;

}
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "flash.events.EventDispatcher",
        metadata: {
            "": [
                "Event",
                [
                    "name",
                    "timer",
                    "type",
                    "flash.events.TimerEvent"
                ],
                "Event",
                [
                    "name",
                    "timerComplete",
                    "type",
                    "flash.events.TimerEvent"
                ]
            ]
        },
        constructor: Timer$,
        reset: reset,
        start: start,
        stop: stop,
        tick$2: tick,
        checkComplete$2: checkComplete,
        timer$2: 0,
        _delay$2: NaN,
        _repeatCount$2: 0,
        _currentCount$2: 0,
        __accessors__: {
            currentCount: {
                get: get$currentCount
            },
            delay: {
                get: get$delay,
                set: set$delay
            },
            repeatCount: {
                get: get$repeatCount,
                set: set$repeatCount
            },
            running: {
                get: get$running
            }
        },
        requires: [
            "AS3.trace",
            "flash.events.EventDispatcher",
            "flash.events.IEventDispatcher"
        ],
        uses: [
            "flash.events.TimerEvent",
            "flash.utils.clearInterval",
            "flash.utils.setInterval"
        ]
    };
});

Ext.define("flash.utils.applyWithRestParameters", function(applyWithRestParameters) {
    /*package flash.utils {

internal*/
    function applyWithRestParameters(closure, /*:Function*/
    parameters) /*:Array*/
    /*:Function*/
    {
        switch (parameters.length) {
            case 0:
                return closure;
            case 1:
                return function() /*:**/
                {
                    return closure(parameters[0]);
                };
            case 2:
                return function() /*:**/
                {
                    return closure(parameters[0], parameters[1]);
                };
            case 3:
                return function() /*:**/
                {
                    return closure(parameters[0], parameters[1], parameters[2]);
                };
            default:
                return function() /*:**/
                {
                    closure.apply(null, parameters);
                };
        }
    }
    /*

}

============================================== Jangaroo part ==============================================*/
    return {
        __factory__: function() {
            return applyWithRestParameters;
        }
    };
});

Ext.define("flash.utils.clearInterval", function(clearInterval) {
    /*package flash.utils {

/**
 * Cancels a specified <code>setInterval()</code> call.
 * @param id The ID of the <code>setInterval()</code> call, which you set to a variable, as in the following:
 *
 * @see #setInterval
 *
 * @example The following example uses the <code>setInterval()</code> method to create a timed interval, calling the <code>myRepeatingFunction()</code> method after regular intervals of one second.
 * <p>Each call of the <code>myRepeatingFunction</code> method increments the <code>counter</code> property, and when it equals the <code>stopCount</code> property, the <code>clearInterval()</code> method is called with the property <code>intervalId</code> which is a reference id to the interval that was created earlier.</p>
 * <listing>
 * package {
 *     import flash.display.Sprite;
 *     import flash.utils.*;
 *
 *     public class ClearIntervalExample extends Sprite {
 *         private var intervalDuration:Number = 1000; // duration between intervals, in milliseconds
 *         private var intervalId:uint;
 *         private var counter:uint     = 0;
 *         private var stopCount:uint     = 3;
 *
 *         public function ClearIntervalExample() {
 *             intervalId = setInterval(myRepeatingFunction, intervalDuration, "Hello", "World");
 *         }
 *
 *         public function myRepeatingFunction():void {
 *             trace(arguments[0] + " " + arguments[1]);
 *
 *             counter++;
 *             if(counter == stopCount) {
 *                 trace("Clearing Interval");
 *                 clearInterval(intervalId);
 *             }
 *         }
 *     }
 * }
 * </listing>
 * /
public*/
    function clearInterval(id) /*:uint*/
    /*:void*/
    {
        window.clearInterval(id);
    }
    /*
}

============================================== Jangaroo part ==============================================*/
    return {
        __factory__: function() {
            return clearInterval;
        }
    };
});

Ext.define("flash.utils.clearTimeout", function(clearTimeout) {
    /*package flash.utils {

/**
 * Cancels a specified <code>setTimeout()</code> call.
 * @param id The ID of the <code>setTimeout()</code> call, which you set to a variable, as in the following:
 *
 * @see #setTimeout
 *
 * @example The following example uses the <code>setTimeout()</code> method to call another method following a specified delay period.
 * <p>A loop is created to count to one million. If the system can process this request faster than a second can expire, <code>clearTimeout()</code> will remove the <code>setTimeout()</code> request, and <code>myDelayedFunction()</code> will not be called.</p>
 * <listing>
 * package {
 *     import flash.display.Sprite;
 *     import flash.utils.*;
 *
 *     public class ClearTimeoutExample extends Sprite {
 *         private var delay:Number = 1000; // delay before calling myDelayedFunction
 *         private var intervalId:uint;
 *         private var count:uint = 1000000;
 *
 *         public function ClearTimeoutExample() {
 *             intervalId = setTimeout(myDelayedFunction, delay);
 *             startCounting();
 *         }
 *
 *         public function startCounting():void {
 *             var i:uint = 0;
 *             do {
 *                 if(i == count-1) {
 *                     clearTimeout(intervalId);
 *                     trace("Your computer can count to " + count + " in less than " + delay/1000 + " seconds.");
 *                 }
 *                 i++;
 *             } while(i &lt; count)
 *         }
 *
 *         public function myDelayedFunction():void {
 *             trace("Time expired.");
 *         }
 *     }
 * }
 * </listing>
 * /
public*/
    function clearTimeout(id) /*:uint*/
    /*:void*/
    {
        window.clearTimeout(id);
    }
    /*
}

============================================== Jangaroo part ==============================================*/
    return {
        __factory__: function() {
            return clearTimeout;
        }
    };
});

Ext.define("flash.utils.describeType", function(describeType) {
    /*package flash.utils {

/**
 * Produces an XML object that describes the ActionScript object named as the parameter of the method. This method implements the programming concept of <i>reflection</i> for the ActionScript language.
 * <p>If the <code>value</code> parameter is an instance of a type, the returned XML object includes all the instance properties of that type, but does not include any static properties. You can check for this condition when you parse the XML object by examining the value of the <code>&lt;type></code> tag's <code>isStatic</code> attribute, which is <code>false</code> when the <code>value</code> parameter is an instance of a type.</p>
 * <p>To obtain the static properties of a type, pass the type itself for the <code>value</code> parameter. The returned XML object includes not only the type's static properties, but also all of its instance properties. The instance properties are nested inside a tag named <code>&lt;factory></code> to distinguish them from the static properties. In this case, the <code>isStatic</code> attribute of the <code>&lt;type></code> tag is <code>true</code>.</p>
 * <p><b>Note:</b> If you need only to traverse an object's inheritance hierarchy and do not need the other information provided by <code>describeType()</code>, use the <code>getQualifiedClassName()</code> and <code>getQualifiedSuperclassName()</code> functions instead.</p>
 * <p>The following table describes some of the tags and attributes of the XML object generated by <code>describeType()</code> (all class and interface names returned are in fully qualified format):</p>
 * <table>
 * <tr><th>Tag</th><th>Attribute</th><th>Description</th></tr>
 * <tr>
 * <td><code>&lt;type></code></td>
 * <td>�</td>
 * <td>The root tag of the XML object.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>name</td>
 * <td>The name of the ActionScript object's data type.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>base</td>
 * <td>The immediate superclass of the ActionScript object's defining class. If the ActionScript object is a class object, the value is <code>Class</code>.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>isDynamic</td>
 * <td><code>true</code> if the ActionScript object's defining class is dynamic; <code>false</code> otherwise. If the ActionScript object is a class object, the value is <code>true</code> because the Class class is dynamic.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>isFinal</td>
 * <td><code>true</code> if the ActionScript object's defining class is final; <code>false</code> otherwise.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>isStatic</td>
 * <td><code>true</code> if the ActionScript object is a class object or constructor function; <code>false</code> otherwise. This attribute is named <code>isStatic</code> because if it is <code>true</code>, any tags that are not nested inside the <code>factory</code> tag are static.</td></tr>
 * <tr>
 * <td><code>&lt;extendsClass></code></td>
 * <td>�</td>
 * <td>There is a separate <code>extendsClass</code> tag for each superclass of the ActionScript object's defining class.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>type</td>
 * <td>The name of a superclass that the ActionScript object's defining class extends.</td></tr>
 * <tr>
 * <td><code>&lt;implementsInterface></code></td>
 * <td>�</td>
 * <td>There is a separate <code>implementsInterface</code> tag for each interface implemented by the ActionScript object's defining class or any of its superclasses.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>type</td>
 * <td>The name of an interface that the ActionScript object's defining class implements.</td></tr>
 * <tr>
 * <td><code>&lt;accessor></code></td>
 * <td>�</td>
 * <td>An accessor is a property defined by getter and setter functions.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>name</td>
 * <td>The name of the accessor.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>access</td>
 * <td>The access rights of the property. Possible values include <code>readonly</code>, <code>writeonly</code>, and <code>readwrite</code>.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>type</td>
 * <td>The data type of the property.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>declaredBy</td>
 * <td>The class that contains the associated getter or setter functions.</td></tr>
 * <tr>
 * <td><code>&lt;constant></code></td>
 * <td>�</td>
 * <td>A constant is a property defined with the <code>const</code> statement.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>name</td>
 * <td>The name of the constant.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>type</td>
 * <td>The data type of the constant.</td></tr>
 * <tr>
 * <td><code>&lt;method></code></td>
 * <td>�</td>
 * <td>A method is a function declared as part of a class definition.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>name</td>
 * <td>The name of the method.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>declaredBy</td>
 * <td>The class that contains the method definition.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>returnType</td>
 * <td>The data type of the method's return value.</td></tr>
 * <tr>
 * <td><code>&lt;parameter></code></td>
 * <td>�</td>
 * <td>There is a separate <code>parameter</code> tag for each parameter that a method defines. This tag is always nested inside a <code>&lt;method></code> tag.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>index</td>
 * <td>A number corresponding to the order in which the parameter appears in the method's parameter list. The first parameter has a value of 1.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>type</td>
 * <td>The data type of the parameter.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>optional</td>
 * <td><code>true</code> if the parameter is optional; <code>false</code> otherwise.</td></tr>
 * <tr>
 * <td><code>&lt;variable></code></td>
 * <td>�</td>
 * <td>A variable is a property defined with the <code>var</code> statement.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>name</td>
 * <td>The name of the variable.</td></tr>
 * <tr>
 * <td>�</td>
 * <td>type</td>
 * <td>The data type of the variable.</td></tr>
 * <tr>
 * <td><code>&lt;factory></code></td>
 * <td>�</td>
 * <td>If the ActionScript object is a class object or constructor function, all instance properties and methods are nested inside this tag. If the <code>isStatic</code> attribute of the <code>&lt;type></code> tag is <code>true</code>, all properties and methods that are not nested within the <code>&lt;factory></code> tag are static. This tag appears only if the ActionScript object is a class object or constructor function.</td></tr></table>
 * @param value The object for which a type description is desired. Any ActionScript value may be passed to this method including all available ActionScript types, object instances, primitive types such as uint, and class objects.
 *
 * @return An XML object containing details about the object that was passed in as a parameter. It provides the following information about the object:
 * <ul>
 * <li>The class of the object</li>
 * <li>The attributes of the class</li>
 * <li>The inheritance tree from the class to its base classes</li>
 * <li>The interfaces implemented by the class</li>
 * <li>The declared instance properties of the class</li>
 * <li>The declared static properties of the class</li>
 * <li>The instance methods of the class</li>
 * <li>The static methods of the class</li>
 * <li>For each method of the class, the name, number of parameters, return type, and parameter types</li></ul>
 * <p><b>Note:</b> <code>describeType()</code> only shows public properties and methods, and will not show properties and methods that are private, package internal or in custom namespaces.</p>
 *
 * @see #getQualifiedClassName()
 * @see #getQualifiedSuperclassName()
 *
 * @example
 * <pre>
 * package {
 *     import flash.display.Sprite;
 *     import flash.utils.describeType;
 *
 *     public class DescribeTypeExample extends Sprite {
 *         public function DescribeTypeExample() {
 *             var child:Sprite = new Sprite();
 *             var description:XML = describeType(child);
 *             trace(description..accessor.&#64;name.toXMLString());
 *         }
 *     }
 * }
 * </pre>
 * /
public*/
    function describeType(value) /*:**/
    /*:XML*/
    {
        var type = /*:Class*/
            value.$isClass ? value : value.self;
        // fake collection:
        var len = /*:uint*/
            0;
        var methods = /*:Object*/
            {
                length: function() /*:uint*/
                {
                    return len;
                }
            };
        if (type && type.prototype) {
            var propertyNames = /*:Array*/
                Object.getOwnPropertyNames(type.prototype);
            for (/* each*/
            var $1 = 0; $1 < /* in*/
            propertyNames.length; ++$1) {
                var p = /*:String*/
                    propertyNames[$1];
                if (p.match(/^[a-zA-Z_]/)) {
                    var descriptor = /*:**/
                        Object.getOwnPropertyDescriptor(type.prototype, p);
                    if (typeof descriptor.value === "function") {
                        methods[len++] = p;
                    }
                }
            }
        }
        return {
            attribute: function(attr) /*:String*/
            /*:String*/
            {
                return attr == "name" ? flash.utils.getQualifiedClassName(value) : null;
            },
            method: {
                "@name": methods
            }
        };
    }
    /*
}

============================================== Jangaroo part ==============================================*/
    return {
        __factory__: function() {
            return describeType;
        },
        uses: [
            "flash.utils.getQualifiedClassName"
        ]
    };
});

Ext.define("flash.utils.getDefinitionByName", function(getDefinitionByName) {
    /*package flash.utils {
import joo.getQualifiedObject;

/**
 * Returns a reference to the class object of the class specified by the <code>name</code> parameter.
 * @param name The name of a class.
 *
 * @return Returns a reference to the class object of the class specified by the <code>name</code> parameter.
 *
 * @throws ReferenceError No public definition exists with the specified name.
 *
 * @example The following example uses the class <code>GetDefinitionByNameExample</code> to create an orange square on the stage. This is accomplished using the following steps: <ol>
 * <li>Variables for the background color of orange and size of 80 pixels are declared, which will later be used in drawing the square.</li>
 * <li>Within the constructor, a variable <code>ClassReference</code> of type Class is assigned to Sprite.</li>
 * <li>An instance of ClassReference called <code>instance</code> is instantiated.</li>
 * <li>Since <code>instance</code> is, by reference, a Sprite object, a square can be drawn and added to the display list using the methods available to Sprite.</li></ol>
 * <listing>
 * package {
 *     import flash.display.DisplayObject;
 *     import flash.display.Sprite;
 *     import flash.utils.getDefinitionByName;
 *
 *     public class GetDefinitionByNameExample extends Sprite {
 *         private var bgColor:uint = 0xFFCC00;
 *         private var size:uint = 80;
 *
 *         public function GetDefinitionByNameExample() {
 *             var ClassReference:Class = getDefinitionByName("flash.display.Sprite") as Class;
 *             var instance:Object = new ClassReference();
 *             instance.graphics.beginFill(bgColor);
 *             instance.graphics.drawRect(0, 0, size, size);
 *             instance.graphics.endFill();
 *             addChild(DisplayObject(instance));
 *         }
 *     }
 * }
 * </listing>
 * /
public*/
    function getDefinitionByName(name) /*:String*/
    /*:Object*/
    {
        var clazz = /*:**/
            joo.getQualifiedObject(name.replace("::", "."));
        if (typeof clazz !== 'function') {
            throw new ReferenceError(name);
        }
        return clazz;
    }
    /*

}

============================================== Jangaroo part ==============================================*/
    return {
        __factory__: function() {
            return getDefinitionByName;
        },
        requires: [
            "AS3.Error"
        ]
    };
});

Ext.define("flash.utils.getQualifiedClassName", function(getQualifiedClassName) {
    /*package flash.utils {

/**
 * Returns the fully qualified class name of an object.
 * @param value The object for which a fully qualified class name is desired. Any ActionScript value may be passed to this method including all available ActionScript types, object instances, primitive types such as uint, and class objects.
 *
 * @return A string containing the fully qualified class name.
 *
 * @see #describeType
 * @see #getQualifiedSuperclassName
 * /
public*/
    function getQualifiedClassName(value) /*:**/
    /*:String*/
    {
        var className = /*:String*/
            value.isInstance || value.$isClass ? value.$className : value.xclass;
        return className.replace(/^(.*)\.([^\.]+$)/, "$1::$2");
    }
    /*
}

============================================== Jangaroo part ==============================================*/
    return {
        __factory__: function() {
            return getQualifiedClassName;
        }
    };
});

Ext.define("flash.utils.getQualifiedSuperclassName", function(getQualifiedSuperclassName) {
    /*package flash.utils {

/**
 * Returns the fully qualified class name of the base class of the object specified by the <code>value</code> parameter. This function provides a quicker way of retrieving the base class name than <code>describeType()</code>, but also doesn't provide all the information <code>describeType()</code> does.
 * <p>After you retrieve the name of a class with this function, you can convert the class name to a class reference with the <code>getDefinitionByName()</code> function.</p>
 * <p><b>Note:</b> This function restricts itself to instance hierarchies, whereas the <code>describeType()</code> function uses class object hierarchies if the <code>value</code> parameter is a data type. Calling <code>describeType()</code> on a data type returns the superclass based on the class object hierarchy, in which all class objects inherit from Class. The <code>getQualifiedSuperclassName()</code> function, however, ignores the class object hierarchy and returns the superclass based on the more familiar instance hierarchy. For example, calling <code>getQualifiedSuperclassName(String)</code> returns <code>Object</code> although technically the String class object inherits from Class. In other words, the results are the same whether you use an instance of a type or the type itself.</p>
 * @param value Any value.
 *
 * @return A fully qualified base class name, or <code>null</code> if none exists.
 *
 * @see #describeType
 * @see #getDefinitionByName
 * @see #getQualifiedSuperclassName
 *
 * /
public*/
    function getQualifiedSuperclassName(value) /*:**/
    /*:String*/
    {
        var type = /*:Function*/
            typeof value == "function" ? value : value.constructor;
        return typeof type["$class"] === "object" ? type["$class"]["superClassDeclaration"].qName.replace(/\.([^\.]+$)/, "::$1") : String(type);
    }
    /*
}

============================================== Jangaroo part ==============================================*/
    return {
        __factory__: function() {
            return getQualifiedSuperclassName;
        }
    };
});

Ext.define("flash.utils.getTimer", {
    __factory__: function() {
        var startTime = new Date().getTime();
        return function() {
            return new Date().getTime() - startTime;
        };
    }
});

Ext.define("flash.utils.setInterval", function(setInterval) {
    /*package flash.utils {

/**
 * Runs a function at a specified interval (in milliseconds).
 * <p>Instead of using the <code>setInterval()</code> method, consider creating a Timer object, with the specified interval, using 0 as the <code>repeatCount</code> parameter (which sets the timer to repeat indefinitely).</p>
 * <p>If you intend to use the <code>clearInterval()</code> method to cancel the <code>setInterval()</code> call, be sure to assign the <code>setInterval()</code> call to a variable (which the <code>clearInterval()</code> function will later reference). If you do not call the <code>clearInterval()</code> function to cancel the <code>setInterval()</code> call, the object containing the set timeout closure function will not be garbage collected.</p>
 * @param closure The name of the function to execute. Do not include quotation marks or parentheses, and do not specify parameters of the function to call. For example, use <code>functionName</code>, not <code>functionName()</code> or <code>functionName(param)</code>.
 * @param delay The interval, in milliseconds.
 * @param rest An optional list of arguments that are passed to the closure function.
 *
 * @return Unique numeric identifier for the timed process. Use this identifier to cancel the process, by calling the <code>clearInterval()</code> method.
 *
 * @see #clearInterval
 *
 * @example The following example uses the <code>setInterval()</code> method to create a timed interval, calling the <code>myRepeatingFunction()</code> method after regular intervals of one second.
 * <listing>
 * package {
 *     import flash.display.Sprite;
 *     import flash.utils.*;
 *
 *     public class SetIntervalExample extends Sprite {
 *         private var intervalDuration:Number = 1000; // duration between intervals, in milliseconds
 *
 *         public function SetIntervalExample() {
 *             var intervalId:uint = setInterval(myRepeatingFunction, intervalDuration, "Hello", "World");
 *         }
 *
 *         public function myRepeatingFunction():void {
 *             trace(arguments[0] + " " + arguments[1]);
 *         }
 *     }
 * }
 * </listing>
 * /
public*/
    function setInterval(closure, /*:Function*/
    delay) /*:Number, ...rest*/
    /*:uint*/
    {
        var rest = Array.prototype.slice.call(arguments, 2);
        var timeout = /*:**/
            window.setInterval(flash.utils.applyWithRestParameters(closure, rest), delay);
        return timeout;
    }
    /*
}

============================================== Jangaroo part ==============================================*/
    return {
        __factory__: function() {
            return setInterval;
        },
        uses: [
            "flash.utils.applyWithRestParameters"
        ]
    };
});

Ext.define("flash.utils.setTimeout", function(setTimeout) {
    /*package flash.utils {

/**
 * Runs a specified function after a specified delay (in milliseconds).
 * <p>Instead of using this method, consider creating a Timer object, with the specified interval, using 1 as the <code>repeatCount</code> parameter (which sets the timer to run only once).</p>
 * <p>If you intend to use the <code>clearTimeout()</code> method to cancel the <code>setTimeout()</code> call, be sure to assign the <code>setTimeout()</code> call to a variable (which the <code>clearTimeout()</code> function will later reference). If you do not call the <code>clearTimeout()</code> function to cancel the <code>setTimeout()</code> call, the object containing the set timeout closure function will not be garbage collected.</p>
 * @param closure The name of the function to execute. Do not include quotation marks or parentheses, and do not specify parameters of the function to call. For example, use <code>functionName</code>, not <code>functionName()</code> or <code>functionName(param)</code>.
 * @param delay The delay, in milliseconds, until the function is executed.
 * @param rest An optional list of arguments that are passed to the closure function.
 *
 * @return Unique numeric identifier for the timed process. Use this identifier to cancel the process, by calling the <code>clearTimeout()</code> method.
 *
 * @see #clearTimeout
 *
 * @example
 * The following example uses the <code>setTimeout()</code> method to call another method following a specified delay period.
 * <listing>
 * package {
 *     import flash.display.Sprite;
 *     import flash.utils.*;
 *
 *     public class SetTimeoutExample extends Sprite {
 *         private var delay:Number = 1000; // delay before calling myDelayedFunction
 *
 *         public function SetTimeoutExample() {
 *             var intervalId:uint = setTimeout(myDelayedFunction, delay, "Hello", "World");
 *         }
 *
 *         public function myDelayedFunction():void {
 *             trace(arguments[0] + " " + arguments[1]);
 *         }
 *     }
 * }
 * </listing>
 * /
public*/
    function setTimeout(closure, /*:Function*/
    delay) /*:Number, ...rest*/
    /*:uint*/
    {
        var rest = Array.prototype.slice.call(arguments, 2);
        var timeout = /*:**/
            window.setTimeout(flash.utils.applyWithRestParameters(closure, rest), delay);
        return timeout;
    }
    /*
}

============================================== Jangaroo part ==============================================*/
    return {
        __factory__: function() {
            return setTimeout;
        },
        uses: [
            "flash.utils.applyWithRestParameters"
        ]
    };
});

